// XHyperLink.cpp  Version 1.0
//
// XHyperLink static control. Will open the default browser with the given URL
// when the user clicks on the link.
//
// Copyright (C) 1997 - 1999 Chris Maunder
// All rights reserved. May not be sold for profit.
//
// Thanks to P? K. T?der for auto-size and window caption changes.
//
// "GotoURL" function by Stuart Patterson
// As seen in the August, 1997 Windows Developer's Journal.
// Copyright 1997 by Miller Freeman, Inc. All rights reserved.
// Modified by Chris Maunder to use TCHARs instead of chars.
//
// "Default hand cursor" from Paul DiLascia's Jan 1998 MSJ article.
//
// 2/29/00 -- P. Shaffer standard font mod.
//
///////////////////////////////////////////////////////////////////////////////
//
// Modified by:  Hans Dietrich
//               hdietrich2@hotmail.com
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "XHyperLink.h"
#include "atlconv.h"		// for Unicode conversion
#include <comip.h> // com_ptr_t
#include <comdef.h> // com_ptr_t
#include <ExDisp.h> // ShellWindows
#include <ShlGuid.h> // SID_SShellBrowser

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TOOLTIP_ID 1
static IShellWindowsPtr g_pShellwindows(NULL);

// Uncomment following line to enable error message box for URL navigation
//#define XHYPERLINK_REPORT_ERROR


#ifndef IDC_HAND
#define IDC_HAND MAKEINTRESOURCE(32649)   // From WINUSER.H
#endif

// sends message to parent when hyperlink is clicked (see SetNotifyParent())
UINT WM_XHYPERLINK_CLICKED = ::RegisterWindowMessage(_T("WM_XHYPERLINK_CLICKED"));

///////////////////////////////////////////////////////////////////////////////
// CXHyperLink

BEGIN_MESSAGE_MAP(CXHyperLink, CStatic)
	//{{AFX_MSG_MAP(CXHyperLink)
	ON_WM_CTLCOLOR_REFLECT()
	ON_WM_SETCURSOR()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_CONTROL_REFLECT(STN_CLICKED, OnClicked)
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////
// ctor
CXHyperLink::CXHyperLink()
{
	m_hLinkCursor     = NULL;			// No cursor as yet
	m_crLinkColour    = RGB(0,0,238);	// Blue
	m_crVisitedColour = RGB(85,26,139);	// Purple
	m_crHoverColour	  = RGB(255,0,0);	// Red
	m_bOverControl    = FALSE;			// Cursor not yet over control
	m_bVisited        = FALSE;			// Hasn't been visited yet.
	m_nUnderline      = ulHover;		// Underline the link?
	m_bAdjustToFit    = TRUE;			// Resize the window to fit the text?
	m_strURL          = _T("");
	m_nTimerID        = 100;
	m_bNotifyParent   = FALSE;			// TRUE = notify parent
	m_bIsURLEnabled   = TRUE;			// TRUE = navigate to url
	m_bToolTip        = TRUE;			// TRUE = display tooltip
	m_crBackground    = (UINT) -1;		// set to default (no bg color)
	m_bAlwaysOpenNew  = FALSE;			// TRUE = always open new browser window

	g_pShellwindows = NULL;
}

///////////////////////////////////////////////////////////////////////////////
// dtor
CXHyperLink::~CXHyperLink()
{
	TRACE(_T("in CXHyperLink::~CXHyperLink\n"));

	if (m_hLinkCursor)
		DestroyCursor(m_hLinkCursor);
	m_hLinkCursor = NULL;
	m_UnderlineFont.DeleteObject();
	if (m_Brush.GetSafeHandle())
		m_Brush.DeleteObject();

	if (NULL != g_pShellwindows)
	{
		ReleaseInstance();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CXHyperLink overrides

///////////////////////////////////////////////////////////////////////////////
// DestroyWindow
BOOL CXHyperLink::DestroyWindow() 
{
	KillTimer(m_nTimerID);
	return CStatic::DestroyWindow();
}

///////////////////////////////////////////////////////////////////////////////
// PreTranslateMessage
BOOL CXHyperLink::PreTranslateMessage(MSG* pMsg) 
{
	m_ToolTip.RelayEvent(pMsg);
	return CStatic::PreTranslateMessage(pMsg);
}

///////////////////////////////////////////////////////////////////////////////
// PreSubclassWindow
void CXHyperLink::PreSubclassWindow() 
{
	// We want to get mouse clicks via STN_CLICKED
	DWORD dwStyle = GetStyle();
	::SetWindowLong(GetSafeHwnd(), GWL_STYLE, dwStyle | SS_NOTIFY);
	
	// Set the URL as the window text
	if (m_strURL.IsEmpty())
		GetWindowText(m_strURL);

	// Check that the window text isn't empty. If it is, set it as the URL.
	CString strWndText;
	GetWindowText(strWndText);
	if (strWndText.IsEmpty()) 
	{
		ASSERT(!m_strURL.IsEmpty());	// Window and URL both NULL. DUH!
		SetWindowText(m_strURL);
	}

	CFont* pFont = GetFont();
	if (!pFont)
	{
		HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
		if (hFont == NULL)
			hFont = (HFONT) GetStockObject(ANSI_VAR_FONT);
		if (hFont)
			pFont = CFont::FromHandle(hFont);
	}

	// Create the underline font
	LOGFONT lf;
	if(pFont)
	{	pFont->GetLogFont(&lf);
		m_StdFont.CreateFontIndirect(&lf);
		lf.lfUnderline = (BYTE) TRUE;
		m_UnderlineFont.CreateFontIndirect(&lf);
	}

	PositionWindow();		// Adjust size of window to fit URL if necessary
	SetDefaultCursor();		// Try and load up a "hand" cursor
	SetUnderline();

	// Create the tooltip
	if (m_bToolTip)
	{
		m_ToolTip.Create(this);
		//-----------------------------------------------------------------------------
		// by cscho
		m_ToolTip.AddTool(this, m_strURL);
		//-----------------------------------------------------------------------------
	}

	CStatic::PreSubclassWindow();
}

/////////////////////////////////////////////////////////////////////////////
// CXHyperLink message handlers

///////////////////////////////////////////////////////////////////////////////
// OnClicked
void CXHyperLink::OnClicked()
{
	m_bOverControl = FALSE;
	int result = HINSTANCE_ERROR + 1;
	if (m_bIsURLEnabled)
		result = (int)GotoURL(m_strURL, SW_SHOW, m_bAlwaysOpenNew);
	m_bVisited = (result > HINSTANCE_ERROR);
	if (!m_bVisited)
	{
		MessageBeep(MB_ICONEXCLAMATION);	 // Unable to follow link
		ReportError(result);
	}
	else 
		SetVisited();						// Repaint to show visited colour

	NotifyParent();
}

///////////////////////////////////////////////////////////////////////////////
// CtlColor
HBRUSH CXHyperLink::CtlColor(CDC* pDC, UINT nCtlColor) 
{
	if (m_bOverControl)
		pDC->SetTextColor(m_crHoverColour);
	else if (m_bVisited)
		pDC->SetTextColor(m_crVisitedColour);
	else
		pDC->SetTextColor(m_crLinkColour);

	// transparent text.
	pDC->SetBkMode(TRANSPARENT);

	if (m_Brush.GetSafeHandle())
	{
		pDC->SetBkColor(m_crBackground);
		return (HBRUSH) m_Brush;
	}
	else
	{
		return (HBRUSH)GetStockObject(NULL_BRUSH);
	}
}

///////////////////////////////////////////////////////////////////////////////
// OnMouseMove
void CXHyperLink::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (!m_bOverControl)		// Cursor has just moved over control
	{
		m_bOverControl = TRUE;

		if (m_nUnderline == ulHover)
			SetFont(&m_UnderlineFont);
		Invalidate();

		SetTimer(m_nTimerID, 100, NULL);
	}
	CStatic::OnMouseMove(nFlags, point);
}

///////////////////////////////////////////////////////////////////////////////
// OnTimer
void CXHyperLink::OnTimer(UINT nIDEvent) 
{
	CPoint p(GetMessagePos());
	ScreenToClient(&p);

	CRect rect(0, 0, 0, 0);
	GetClientRect(rect);
	if (!rect.PtInRect(p))
	{
		m_bOverControl = FALSE;
		KillTimer(m_nTimerID);

		if (m_nUnderline != ulAlways)
			SetFont(&m_StdFont);
		rect.bottom+=10;
		InvalidateRect(rect);
	}
	
	CStatic::OnTimer(nIDEvent);
}

///////////////////////////////////////////////////////////////////////////////
// OnSetCursor
BOOL CXHyperLink::OnSetCursor(CWnd* /*pWnd*/, UINT /*nHitTest*/, UINT /*message*/) 
{
	if (m_hLinkCursor)
	{
		::SetCursor(m_hLinkCursor);
		return TRUE;
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
// OnEraseBkgnd
BOOL CXHyperLink::OnEraseBkgnd(CDC* pDC) 
{
	CRect rect(0, 0, 0, 0);
	GetClientRect(rect);
	if (m_crBackground != (UINT)-1)
		pDC->FillSolidRect(rect, m_crBackground);
	else
		pDC->FillSolidRect(rect, ::GetSysColor(COLOR_3DFACE));

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CXHyperLink operations

///////////////////////////////////////////////////////////////////////////////
// SetURL
void CXHyperLink::SetURL(CString strURL)
{
	m_strURL = strURL;

	if (::IsWindow(GetSafeHwnd())) 
	{
		PositionWindow();
	}
}

///////////////////////////////////////////////////////////////////////////////
// SetColours
void CXHyperLink::SetColours(COLORREF crLinkColour, 
							 COLORREF crVisitedColour,	
							 COLORREF crHoverColour /* = -1 */) 
{ 
	m_crLinkColour	= crLinkColour; 
	m_crVisitedColour = crVisitedColour;

	if (crHoverColour == -1)
		m_crHoverColour = ::GetSysColor(COLOR_HIGHLIGHT);
	else
		m_crHoverColour = crHoverColour;

	if (::IsWindow(m_hWnd))
		Invalidate(); 
}

///////////////////////////////////////////////////////////////////////////////
// SetBackgroundColour
void CXHyperLink::SetBackgroundColour(COLORREF crBackground)
{
	m_crBackground = crBackground;
	if (m_Brush.GetSafeHandle())
		m_Brush.DeleteObject();
	m_Brush.CreateSolidBrush(m_crBackground);
}

///////////////////////////////////////////////////////////////////////////////
// SetVisited
void CXHyperLink::SetVisited(BOOL bVisited /* = TRUE */) 
{ 
	m_bVisited = bVisited; 

	if (::IsWindow(GetSafeHwnd()))
		Invalidate(); 
}

///////////////////////////////////////////////////////////////////////////////
// SetLinkCursor
void CXHyperLink::SetLinkCursor(HCURSOR hCursor)
{ 
	m_hLinkCursor = hCursor;
	if (m_hLinkCursor == NULL)
		SetDefaultCursor();
}

///////////////////////////////////////////////////////////////////////////////
// SetUnderline
void CXHyperLink::SetUnderline(int nUnderline /*=ulHover*/)
{
	if (m_nUnderline == nUnderline)
		return;

	if (::IsWindow(GetSafeHwnd()))
	{
		if (nUnderline == ulAlways)
			SetFont(&m_UnderlineFont);
		else
			SetFont(&m_StdFont);

		Invalidate(); 
	}

	m_nUnderline = nUnderline;
}

///////////////////////////////////////////////////////////////////////////////
// SetAutoSize
void CXHyperLink::SetAutoSize(BOOL bAutoSize /* = TRUE */)
{
	m_bAdjustToFit = bAutoSize;

	if (::IsWindow(GetSafeHwnd()))
		PositionWindow();
}

///////////////////////////////////////////////////////////////////////////////
// SetWindowText
void CXHyperLink::SetWindowText(LPCTSTR lpszString)
{
	ASSERT(lpszString);
	if (!lpszString)
		return;
	CStatic::SetWindowText(_T(""));
	RedrawWindow();
	CStatic::SetWindowText(lpszString);
	PositionWindow();
}

///////////////////////////////////////////////////////////////////////////////
// PositionWindow

// Move and resize the window so that the window is the same size
// as the hyperlink text. This stops the hyperlink cursor being active
// when it is not directly over the text. If the text is left justified
// then the window is merely shrunk, but if it is centred or right
// justified then the window will have to be moved as well.
//
// Suggested by P? K. T?der 
//
void CXHyperLink::PositionWindow()
{
	if (!::IsWindow(GetSafeHwnd()) || !m_bAdjustToFit) 
		return;

	// Get the current window position
	CRect WndRect(0, 0, 0, 0), ClientRect(0, 0, 0, 0);
	GetWindowRect(WndRect);
	GetClientRect(ClientRect);

	ClientToScreen(ClientRect);

	CWnd* pParent = GetParent();
	if (pParent)
	{
		pParent->ScreenToClient(WndRect);
		pParent->ScreenToClient(ClientRect);
	}

	// Get the size of the window text
	CString strWndText;
	GetWindowText(strWndText);

	CDC* pDC = GetDC();
	CFont* pOldFont = pDC->SelectObject(&m_UnderlineFont);
	CSize Extent = pDC->GetTextExtent(strWndText);
	pDC->SelectObject(pOldFont);
	ReleaseDC(pDC);

	// Adjust for window borders
	Extent.cx += WndRect.Width() - ClientRect.Width(); 
	Extent.cy += WndRect.Height() - ClientRect.Height(); 

	// Get the text justification via the window style
	DWORD dwStyle = GetStyle();

	// Recalc the window size and position based on the text justification
	if (dwStyle & SS_CENTERIMAGE)
		WndRect.DeflateRect(0, (WndRect.Height() - Extent.cy)/2);
	else
		WndRect.bottom = WndRect.top + Extent.cy;

	if (dwStyle & SS_CENTER) 
		WndRect.DeflateRect((WndRect.Width() - Extent.cx)/2, 0);
	else if (dwStyle & SS_RIGHT) 
		WndRect.left = WndRect.right - Extent.cx;
	else // SS_LEFT = 0, so we can't test for it explicitly 
		WndRect.right = WndRect.left + Extent.cx;

	// Move the window
	SetWindowPos(NULL, 
				 WndRect.left, WndRect.top, 
				 WndRect.Width(), WndRect.Height(), 
				 SWP_NOZORDER);
}

/////////////////////////////////////////////////////////////////////////////
// CXHyperLink implementation

///////////////////////////////////////////////////////////////////////////////
// SetDefaultCursor
void CXHyperLink::SetDefaultCursor()
{
	if (m_hLinkCursor == NULL)				// No cursor handle - try to load one
	{
		// First try to load the Win98 / Windows 2000 hand cursor

		TRACE(_T("loading from IDC_HAND\n"));
		m_hLinkCursor = AfxGetApp()->LoadStandardCursor(IDC_HAND);

		if (m_hLinkCursor == NULL)			// Still no cursor handle - 
											// load the WinHelp hand cursor
		{
			// The following appeared in Paul DiLascia's Jan 1998 MSJ articles.
			// It loads a "hand" cursor from the winhlp32.exe module.

			TRACE(_T("loading from winhlp32\n"));

			// Get the windows directory
			CString strWndDir;
			GetWindowsDirectory(strWndDir.GetBuffer(MAX_PATH), MAX_PATH);
			strWndDir.ReleaseBuffer();

			strWndDir += _T("\\winhlp32.exe");

			// This retrieves cursor #106 from winhlp32.exe, which is a hand pointer
			HMODULE hModule = LoadLibrary(strWndDir);
			if (hModule) 
			{
				HCURSOR hHandCursor = ::LoadCursor(hModule, MAKEINTRESOURCE(106));
				if (hHandCursor)
					m_hLinkCursor = CopyCursor(hHandCursor);
				FreeLibrary(hModule);
			}
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
// ReportError
void CXHyperLink::ReportError(int nError)
{
#ifdef XHYPERLINK_REPORT_ERROR

	CString str;
	switch (nError) 
	{
		case 0:							str = "The operating system is out\nof memory or resources."; break;
		case SE_ERR_PNF:				str = "The specified path was not found."; break;
		case SE_ERR_FNF:				str = "The specified file was not found."; break;
		case ERROR_BAD_FORMAT:			str = "The .EXE file is invalid\n(non-Win32 .EXE or error in .EXE image)."; break;
		case SE_ERR_ACCESSDENIED:		str = "The operating system denied\naccess to the specified file."; break;
		case SE_ERR_ASSOCINCOMPLETE:	str = "The filename association is\nincomplete or invalid."; break;
		case SE_ERR_DDEBUSY:			str = "The DDE transaction could not\nbe completed because other DDE transactions\nwere being processed."; break;
		case SE_ERR_DDEFAIL:			str = "The DDE transaction failed."; break;
		case SE_ERR_DDETIMEOUT:			str = "The DDE transaction could not\nbe completed because the request timed out."; break;
		case SE_ERR_DLLNOTFOUND:		str = "The specified dynamic-link library was not found."; break;
		case SE_ERR_NOASSOC:			str = "There is no application associated\nwith the given filename extension."; break;
		case SE_ERR_OOM:				str = "There was not enough memory to complete the operation."; break;
		case SE_ERR_SHARE:				str = "A sharing violation occurred. ";
		default:						str.Format(_T("Unknown Error (%d) occurred."), nError); break;
	}
	str = "Unable to open hyperlink:\n\n" + str;
	AfxMessageBox(str, MB_ICONEXCLAMATION | MB_OK);

#else

	UNUSED_ALWAYS(nError);

#endif	// XHYPERLINK_REPORT_ERROR
}

///////////////////////////////////////////////////////////////////////////////
// NotifyParent
void CXHyperLink::NotifyParent()
{
	if (m_bNotifyParent)
	{
		CWnd *pParent = GetParent();
		if (pParent && ::IsWindow(pParent->m_hWnd))
		{
			// wParam will contain control id
			pParent->SendMessage(WM_XHYPERLINK_CLICKED, GetDlgCtrlID());
		}
	}
}

bool CXHyperLink::RunApp(CString appName)
{
	STARTUPINFO sinfo;
	FillMemory(&sinfo, sizeof(STARTUPINFO), 0);
	sinfo.cb = sizeof(STARTUPINFO);

	PROCESS_INFORMATION pinfo;
	BSTR bstrString = appName.AllocSysString();
	LPWSTR pwstr = (LPWSTR)bstrString;

	SysFreeString(bstrString);

	if (CreateProcess(NULL, pwstr, NULL, NULL, FALSE,
		NORMAL_PRIORITY_CLASS, NULL,
		NULL, // current dir
		&sinfo,
		&pinfo)
		)
	{
		CloseHandle(pinfo.hProcess);
		CloseHandle(pinfo.hThread);
		return true;
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////////
// GotoURL
HINSTANCE CXHyperLink::GotoURL(LPCTSTR url, int showcmd, BOOL bAlwaysOpenNew /*= FALSE*/)
{
	HINSTANCE result = (HINSTANCE)(HINSTANCE_ERROR + 1);
	
	if (!url || url[0] == _T('\0'))
		return result;

	CString strBrowser = GetDefaultBrowerPath();

	// 1 : ⺻  
	if (!strBrowser.IsEmpty())
	{
		if (RunBrowser(strBrowser, url))	
			return result;
	}

	strBrowser = GetExplorerPath(_T("http\\shell\\open\\command"));

	// 1 : ⺻  
	if (!strBrowser.IsEmpty())
	{
		if (RunBrowser(strBrowser, url))	
			return result;
	}

	// 2 : ͳ ͽ÷η (1 õ н)
	strBrowser = GetExplorerPath(_T("Applications\\iexplore.exe\\shell\\open\\command"));

	if (!strBrowser.IsEmpty())
	{
		if (RunBrowser(strBrowser, url))	
			return result;
	}

	// 3 :  α׷ (2 н)
	TCHAR *verb = _T("open");

	if (bAlwaysOpenNew)
		verb = _T("new");

	result = ShellExecute(NULL, verb, url, NULL,NULL, showcmd);

	if ((UINT)result <= HINSTANCE_ERROR)
	{
		AfxMessageBox(IDS_STRING_MAIN_NOTFOUNDIE);
	}

	return result;
}

HINSTANCE CXHyperLink::SendMailTo(LPCTSTR email, int showcmd)
{
	HINSTANCE result = (HINSTANCE)(HINSTANCE_ERROR + 1);
	CString strMailto;
	strMailto.Format(_T("mailto:%s"), email);

	result = ShellExecute(NULL, _T("open"), strMailto, NULL,NULL, showcmd);

	if ((UINT)result <= HINSTANCE_ERROR)
	{
		// ⺻ ̸  α׷ ã  ٴ  ޽ ʿ.
		//AfxMessageBox(IDS_STRING_MAIN_NOTFOUNDIE);
	}

	return result;
}

void CXHyperLink::SetToolTipText( CString strToolTip )
{
	m_ToolTip.UpdateTipText(strToolTip, this);
}

void CXHyperLink::SetLinkFont(CFont& StdFont)
{
	LOGFONT logFont;
	StdFont.GetLogFont(&logFont);

	if (m_StdFont.GetSafeHandle())
		m_StdFont.Detach();

	m_StdFont.CreateFontIndirect(&logFont);
}

CString CXHyperLink::GetExplorerPath( CString strRegistry )
{
	CString resultPath(_T(""));

	HKEY hKey = NULL;
	if (RegOpenKeyEx(HKEY_CLASSES_ROOT, strRegistry, 0, KEY_READ, &hKey) == ERROR_SUCCESS)
	{
		DWORD cbData = 0;
		if (RegQueryValueEx(hKey, NULL, NULL, NULL, NULL, &cbData) == ERROR_SUCCESS && cbData > 0)
		{
			TCHAR *psz = new TCHAR[cbData];
			if (psz != NULL)
			{
				if (RegQueryValueEx(hKey, NULL, NULL, NULL, (LPBYTE)psz, &cbData) == ERROR_SUCCESS)
				{
					resultPath.Format(_T("%s"), psz);
				}
				delete [] psz;
			}
		}
		RegCloseKey(hKey);
	}

	return resultPath;
}

CString CXHyperLink::GetDefaultBrowerPath()
{
	CString strDefaultBrowerProgId = _T("");

	HKEY hKey = NULL;
	if (RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\Microsoft\\Windows\\Shell\\Associations\\UrlAssociations\\http\\UserChoice"), 0, KEY_READ, &hKey) == ERROR_SUCCESS)
	{
		DWORD cbData = 0;
		if (RegQueryValueEx(hKey, _T("Progid"), NULL, NULL, NULL, &cbData) == ERROR_SUCCESS && cbData > 0)
		{
			TCHAR *psz = new TCHAR[cbData];
			if (psz != NULL)
			{
				if (RegQueryValueEx(hKey, _T("Progid"), NULL, NULL, (LPBYTE)psz, &cbData) == ERROR_SUCCESS)
				{
					strDefaultBrowerProgId.Format(_T("%s"), psz);
				}
				delete [] psz;
			}
		}
		RegCloseKey(hKey);
	}

	CString strDefaultBrowerPath = _T("");
	CString strReg;
	strReg.Format(_T("%s\\shell\\open\\command"), strDefaultBrowerProgId);
	if (RegOpenKeyEx(HKEY_CLASSES_ROOT, strReg, 0, KEY_READ, &hKey) == ERROR_SUCCESS)
	{
		DWORD cbData = 0;
		if (RegQueryValueEx(hKey, NULL, NULL, NULL, NULL, &cbData) == ERROR_SUCCESS && cbData > 0)
		{
			TCHAR *psz = new TCHAR[cbData];
			if (psz != NULL)
			{
				if (RegQueryValueEx(hKey, NULL, NULL, NULL, (LPBYTE)psz, &cbData) == ERROR_SUCCESS)
				{
					strDefaultBrowerPath.Format(_T("%s"), psz);
				}
				delete [] psz;
			}
		}
		RegCloseKey(hKey);
	}

	return strDefaultBrowerPath;
}

BOOL CXHyperLink::UseAlreadyOpenedIE( CString strUrl, CString strExclusiveKeyword )
{
	CString strIE(_T("IEFrame"));

	if(NULL == g_pShellwindows)
	{
		HRESULT hr;

		hr = CoInitialize(NULL);
		if (FAILED(hr))
			return FALSE;

		hr = CoCreateInstance(CLSID_ShellWindows, NULL, CLSCTX_ALL, IID_IShellWindows, (void **)&g_pShellwindows);
		if (FAILED(hr))
		{
			ReleaseInstance();
			return FALSE;
		}
	}

	return FindProperIE(strUrl, strExclusiveKeyword);
}

BOOL CXHyperLink::FindProperIE( CString strUrl, CString strExclusiveKeyword )
{
	VARIANT v;
	V_VT(&v) = VT_I4;
	IDispatchPtr disp(0);

	HRESULT hr;
	for (V_I4(&v) = 0; S_OK == (hr = g_pShellwindows->Item(v, &disp)); V_I4(&v)++)
	{
		if (disp == NULL)
			continue;

		// Cast to IWebBrowser2
		IWebBrowser2Ptr browser(0);
		
		if (S_OK != disp->QueryInterface(IID_IWebBrowser2, (void **)&browser))
			continue;
		
		if (browser == NULL)
			continue;

		HWND hWnd;
		browser->get_HWND((SHANDLE_PTR *)&hWnd);

		IServiceProviderPtr servProv(NULL);
		IOleWindowPtr oleWnd(NULL);
		HWND tab_hwnd;

		if (S_OK != browser->QueryInterface(IID_IServiceProvider, (void **)&servProv))
			continue;

		if (servProv == NULL)
			continue;

		if (S_OK != servProv->QueryService(SID_SShellBrowser, IID_IOleWindow, (void **)&oleWnd))
			continue;

		if (oleWnd == NULL)
			continue;

		oleWnd->GetWindow(&tab_hwnd);

		BSTR uri_b;
		if (S_OK == browser->get_LocationURL(&uri_b))
		{
			CString strCurrentURL(uri_b);

			if (-1 != strCurrentURL.Find(strUrl))
			{
				if (strExclusiveKeyword.IsEmpty() ||
					-1 == strCurrentURL.Find(strExclusiveKeyword))
				{
					//Activate Main
					if (::IsIconic(hWnd))
						::ShowWindow(hWnd, SW_RESTORE);

					::SetForegroundWindow(hWnd);
					browser->Refresh();

					//Activate Tab
					if (tab_hwnd) 
						ActivateTab(tab_hwnd);

					return TRUE;
				}
			}
		}
	}

	return FALSE;
}

void CXHyperLink::ReleaseInstance()
{
	g_pShellwindows = NULL;
	CoUninitialize();
}

void CXHyperLink::ActivateTab( HWND hWnd )
{
	IAccessible *tabsAccess;
	VARIANT tabQueryStruct;

	tabQueryStruct.vt=VT_I4;
	tabQueryStruct.lVal=CHILDID_SELF;

	if(S_OK == AccessibleObjectFromWindow(hWnd, OBJID_WINDOW, IID_IAccessible,(void **)&tabsAccess))
	{
		tabsAccess->accDoDefaultAction(tabQueryStruct);

	}
}

bool CXHyperLink::RunBrowser( CString &strBrowser, LPCTSTR url )
{
	std::string::size_type nStart = strBrowser.Find(_T('"'), 0);

	// if first is a quote, then exe path is in quotes, so find the second quote
	if (nStart == 0)
	{
		std::string::size_type nEnd = strBrowser.Find(_T('"'), 1);
		if (nStart != nEnd && nEnd != std::string::npos)
		{
			strBrowser = strBrowser.Mid(nStart+1, nEnd-nStart-1);
		}
	}
	// otherwise if no quotes, 2nd point is the first space after the last backslash
	else
	{
		std::string::size_type nIndex = strBrowser.ReverseFind(_T('\\'));
		if (nIndex != std::string::npos)
		{
			std::string::size_type nSpace = strBrowser.Find(_T(' '), nIndex);
			if (nSpace != std::string::npos)
			{
				strBrowser = strBrowser.Mid(0, nSpace);
			}
		}
	}

	CString command = _T("\"");
	command += strBrowser;
	command += _T("\" \"");
	command += url;
	command += _T("\"");

	return RunApp(command);
}

